var TestRunner, color, timeout, w,
  __slice = [].slice;

w = require('when');

timeout = require('when/timeout');

color = require('./color');

module.exports = TestRunner = (function() {
  function TestRunner() {
    var didBeep;
    this.done = false;
    this.report = [];
    this.pending = 0;
    didBeep = false;
    this._reporter = require('./simpleErrorReporter');
  }

  TestRunner.prototype.setErrorReporter = function(reporter) {
    return this._reporter = reporter;
  };

  TestRunner.prototype.checkDone = function() {
    var id, item, _i, _len, _ref;
    if (this.pending > 0 || this.done) {
      return;
    }
    this.done = true;
    _ref = this.report;
    for (id = _i = 0, _len = _ref.length; _i < _len; id = ++_i) {
      item = _ref[id];
      if (item.type === 'description') {
        this.renderDescription(id, item);
      } else {
        this.renderTestcase(id, item);
      }
    }
  };

  TestRunner.prototype.renderDescription = function(id, item) {
    if (id > 0) {
      console.log();
    }
    return console.log('       %s', item.name);
  };

  TestRunner.prototype.renderTestcase = function(id, item) {
    if (item.skipped != null) {
      return this.renderSkipped(id, item);
    } else if (item.success) {
      return this.renderSucess(id, item);
    } else {
      return this.renderFailure(id, item);
    }
  };

  TestRunner.prototype.renderSucess = function(id, item) {
    return console.log('     √ \x1b[32m%s\x1b[0m', item.name);
  };

  TestRunner.prototype.renderSkipped = function(id, item) {
    return console.log('     - \x1b[90m%s\x1b[0m', item.name);
  };

  TestRunner.prototype.handleSuccess = function(id, name) {
    this.report[id].success = true;
    return this.checkDone();
  };

  TestRunner.prototype.handleError = function(id, name, err) {
    this.report[id].success = false;
    this.report[id].error = err;
    return this.checkDone();
  };

  TestRunner.prototype.renderFailure = function(id, item) {
    console.log('       ' + color(item.name, 'red') + '\n');
    this._reporter(item.error);
    if (!this.didBeep) {
      console.log("\007");
      return this.didBeep = true;
    }
  };

  TestRunner.prototype.describe = function(name) {
    this.report.push({
      type: 'description',
      name: name
    });
  };

  TestRunner.prototype.skip = function(name) {
    return this.report.push({
      type: 'test',
      name: name,
      skipped: true
    });
  };

  TestRunner.prototype.test = function() {
    var checkTimeout, fn, id, name, promise, rest;
    name = arguments[0], rest = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    this.pending++;
    this.report.push({
      type: 'test',
      name: name,
      result: 'pending'
    });
    id = this.report.length - 1;
    if (rest.length === 1) {
      checkTimeout = true;
      fn = rest[0];
    } else if (rest.length === 2) {
      checkTimeout = Boolean(rest[0]);
      fn = rest[1];
    }
    promise = w().then((function(_this) {
      return function() {
        return fn();
      };
    })(this));
    if (checkTimeout) {
      promise = timeout(promise, 100);
    }
    return promise.then((function(_this) {
      return function() {
        _this.pending--;
        return _this.handleSuccess(id, name);
      };
    })(this), (function(_this) {
      return function(err) {
        _this.pending--;
        return _this.handleError(id, name, err);
      };
    })(this));
  };

  return TestRunner;

})();
